@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -235,7 +235,17 @@ int16_t ping_clamd(const struct optstruct *opts)
         }
 
         if (i + 1 < attempts) {
-            logg("*PINGing again in %" PRIu64 " seconds\n", interval);
+            if (optget(opts, "wait")->enabled) {
+                if (interval == 1)
+                    logg("*Could not connect, will try again in %lu second\n", interval);
+                else
+                    logg("*Could not connect, will try again in %lu seconds\n", interval);
+            } else {
+                if (interval == 1)
+                    logg("Could not connect, will PING again in %lu second\n", interval);
+                else
+                    logg("Could not connect, will PING again in %lu seconds\n", interval);
+            }
             sleep(interval);
         }
         i++;
@@ -243,7 +253,11 @@ int16_t ping_clamd(const struct optstruct *opts)
 
     /* timed out */
     ret = 1;
-    logg("*PING timeout exceeded with no response from clamd\n");
+    if (optget(opts, "wait")->enabled) {
+        logg("Wait timeout exceeded; Could not connect to clamd\n");
+    } else {
+        logg("PING timeout exceeded; No response from clamd\n");
+    }
 
 done:
     if (attempt_str) {
@@ -294,7 +308,21 @@ static char *makeabs(const char *basepath)
 static int client_scan(const char *file, int scantype, int *infected, int *err, int maxlevel, int session, int flags)
 {
     int ret;
-    char *fullpath = makeabs(file);
+    char *real_path = NULL;
+    char *fullpath  = NULL;
+
+    /* Convert relative path to fullpath */
+    fullpath = makeabs(file);
+
+    /* Convert fullpath to the real path (evaluating symlinks and . and ..).
+       Doing this early on will ensure that the scan results will appear consistent
+       across regular scans, --fdpass scans, and --stream scans. */
+    if (CL_SUCCESS != cli_realpath(fullpath, &real_path)) {
+        logg("*client_scan: Failed to determine real filename of %s.\n", fullpath);
+    } else {
+        free(fullpath);
+        fullpath = real_path;
+    }
 
     if (!fullpath)
         return 0;
