@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -151,11 +151,13 @@ static int send_stream(int sockd, const char *filename)
 
     if (filename) {
         if ((fd = safe_open(filename, O_RDONLY | O_BINARY)) < 0) {
-            logg("~%s: Access denied. ERROR\n", filename);
+            logg("~%s: Failed to open file. ERROR\n", filename);
             return 0;
         }
-    } else
+    } else {
+        /* Read stream from STDIN */
         fd = 0;
+    }
 
     if (sendln(sockd, "zINSTREAM", 10)) {
         close(fd);
@@ -199,7 +201,7 @@ static int send_fdpass(int sockd, const char *filename)
 
     if (filename) {
         if ((fd = open(filename, O_RDONLY)) < 0) {
-            logg("~%s: Access denied. ERROR\n", filename);
+            logg("~%s: Failed to open file\n", filename);
             return 0;
         }
     } else
@@ -234,19 +236,32 @@ static int send_fdpass(int sockd, const char *filename)
 /* 0: scan, 1: skip */
 static int chkpath(const char *path)
 {
+    int status = 0;
     const struct optstruct *opt;
+    char *real_path = NULL;
+
+    if (!path) {
+        status = 1;
+        goto done;
+    }
 
     if ((opt = optget(clamdopts, "ExcludePath"))->enabled) {
         while (opt) {
             if (match_regex(path, opt->strarg) == 1) {
                 if (printinfected != 1)
                     logg("~%s: Excluded\n", path);
-                return 1;
+                status = 1;
+                goto done;
             }
             opt = opt->nextarg;
         }
     }
-    return 0;
+
+done:
+    if (NULL != real_path) {
+        free(real_path);
+    }
+    return status;
 }
 
 static int ftw_chkpath(const char *path, struct cli_ftw_cbdata *data)
@@ -429,12 +444,14 @@ static cl_error_t serial_callback(STATBUF *sb, char *filename, const char *path,
     if (reason != visit_directory_toplev) {
         if (CL_SUCCESS != cli_realpath((const char *)path, &real_filename)) {
             logg("*Failed to determine real filename of %s.\n", path);
+            logg("*Quarantine of the file may fail if file path contains symlinks.\n");
         } else {
             path = real_filename;
         }
     }
 
     if (chkpath(path)) {
+        /* Exclude the path */
         status = CL_SUCCESS;
         goto done;
     }
@@ -489,6 +506,9 @@ static cl_error_t serial_callback(STATBUF *sb, char *filename, const char *path,
 
     status = CL_SUCCESS;
 done:
+    if (NULL != real_filename) {
+        free(real_filename);
+    }
     free(filename);
     return status;
 }
@@ -613,13 +633,16 @@ static cl_error_t parallel_callback(STATBUF *sb, char *filename, const char *pat
     if (reason != visit_directory_toplev) {
         if (CL_SUCCESS != cli_realpath((const char *)filename, &real_filename)) {
             logg("*Failed to determine real filename of %s.\n", filename);
+            logg("*Quarantine of the file may fail if file path contains symlinks.\n");
         } else {
-            free(filename);
+            free(filename); /* callback is responsible for free'ing filename parameter. */
             filename = real_filename;
         }
     }
 
     if (chkpath(filename)) {
+        /* Exclude the path */
+        status = CL_SUCCESS;
         goto done;
     }
     c->files++;
