@@ -261,7 +261,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
             ret = FC_ETESTFAIL;
         }
         if (FC_SUCCESS != ret) {
-            logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+            logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
             status = FC_ETESTFAIL;
             goto done;
         }
@@ -276,7 +276,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
             logg("^pipe() failed: %s\n", strerror(errno));
             ret = fc_test_database(dbFilename, fc_context->bBytecodeEnabled);
             if (FC_SUCCESS != ret) {
-                logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                 status = FC_ETESTFAIL;
                 goto done;
             }
@@ -302,7 +302,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
                     /* Test the database without forking. */
                     ret = fc_test_database(dbFilename, fc_context->bBytecodeEnabled);
                     if (FC_SUCCESS != ret) {
-                        logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                        logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                         status = FC_ETESTFAIL;
                         goto done;
                     }
@@ -367,7 +367,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
                     if (WIFEXITED(stat_loc)) {
                         ret = (fc_error_t)WEXITSTATUS(stat_loc);
                         if (FC_SUCCESS != ret) {
-                            logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                            logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                             status = FC_ETESTFAIL;
                             goto done;
                         }
@@ -555,7 +555,7 @@ static void free_string_list(char **stringList, uint32_t nListItems)
 /**
  * @brief Get the database server list object
  *
- * @param opts          Freshclam options struct.
+ * @param opts          FreshClam options struct.
  * @param serverList    [out] List of servers.
  * @param nServers      [out] Number of servers in list.
  * @param bPrivate      [out] Non-zero if PrivateMirror servers were selected.
@@ -760,7 +760,7 @@ static fc_error_t initialize(struct optstruct *opts)
                 if (errno == 0) {
                     logg("Create the \"%s\" user account for freshclam to use, or set the DatabaseOwner config option in freshclam.conf to a different user.\n",
                          optget(opts, "DatabaseOwner")->strarg);
-                    logg("For more information, see https://www.clamav.net/documents/installing-clamav-on-unix-linux-macos-from-source\n");
+                    logg("For more information, see https://docs.clamav.net/manual/Installing/Installing-from-source-Unix.html\n");
                 } else {
                     logg("An unexpected error occurred when attempting to query the \"%s\" user account.\n",
                          optget(opts, "DatabaseOwner")->strarg);
@@ -927,8 +927,24 @@ static fc_error_t initialize(struct optstruct *opts)
         logg("Connecting via %s\n", fcConfig.proxyServer);
     }
 
-    if (optget(opts, "HTTPUserAgent")->enabled)
-        fcConfig.userAgent = optget(opts, "HTTPUserAgent")->strarg;
+    if (optget(opts, "HTTPUserAgent")->enabled) {
+
+        if (!(optget(opts, "PrivateMirror")->enabled) &&
+            (optget(opts, "DatabaseMirror")->enabled) &&
+            (strstr(optget(opts, "DatabaseMirror")->strarg, "clamav.net"))) {
+            /*
+             * Using the official project CDN.
+             */
+            logg("In an effort to reduce CDN data costs, HTTPUserAgent may not be used when updating from clamav.net.\n");
+            logg("The HTTPUserAgent specified in your config will be ignored so that FreshClam is not blocked by the CDN.\n");
+            logg("If ClamAV's user agent is not allowed through your firewall/proxy, please contact your network administrator.\n\n");
+        } else {
+            /*
+             * Using some other CDN or private mirror.
+             */
+            fcConfig.userAgent = optget(opts, "HTTPUserAgent")->strarg;
+        }
+    }
 
     fcConfig.maxAttempts    = optget(opts, "MaxAttempts")->numarg;
     fcConfig.connectTimeout = optget(opts, "ConnectTimeout")->numarg;
@@ -1463,7 +1479,7 @@ fc_error_t perform_database_update(
             (void *)fc_context,
             &nUpdated);
         if (FC_SUCCESS != ret) {
-            logg("!Database update process failed: %s (%d)\n", fc_strerror(ret), ret);
+            logg("!Database update process failed: %s\n", fc_strerror(ret));
             status = ret;
             goto done;
         }
@@ -1480,7 +1496,7 @@ fc_error_t perform_database_update(
             (void *)fc_context,
             &nUpdated);
         if (FC_SUCCESS != ret) {
-            logg("!Database update process failed: %s (%d)\n", fc_strerror(ret), ret);
+            logg("!Database update process failed: %s\n", fc_strerror(ret));
             status = ret;
             goto done;
         }
@@ -1701,16 +1717,6 @@ int main(int argc, char **argv)
             status = ret;
             goto done;
         }
-        if (optget(opts, "SafeBrowsing")->enabled) {
-            if (FC_SUCCESS != (ret = string_list_add("safebrowsing", &optInList, &nOptIns))) {
-                free_string_list(optInList, nOptIns);
-                optInList = NULL;
-
-                mprintf("!Failed to add safebrowsing to list of opt-in databases.\n");
-                status = ret;
-                goto done;
-            }
-        }
 
         /*
          * Collect list of database opt-outs.
@@ -1900,6 +1906,36 @@ int main(int argc, char **argv)
             logFileName = logFileOpt->strarg;
         }
 
+        /*
+         * freshclam may have created the freshclam.dat file with as root
+         * if run in daemon-mode, so we should give ownership to the
+         * DatabaseOwner if we're supposed to drop privileges..
+         */
+        if ((0 == geteuid()) && (NULL != optget(opts, "DatabaseOwner")->strarg)) {
+            struct passwd *user = NULL;
+            STATBUF sb;
+
+            if ((user = getpwnam(optget(opts, "DatabaseOwner")->strarg)) == NULL) {
+                logg("^Can't get information about user %s.\n", optget(opts, "DatabaseOwner")->strarg);
+                fprintf(stderr, "ERROR: Can't get information about user %s.\n", optget(opts, "DatabaseOwner")->strarg);
+                status = FC_ECONFIG;
+                goto done;
+            }
+
+            /*Change ownership of the freshclam DAT file to the user we are going to switch to.*/
+            if (CLAMSTAT("freshclam.dat", &sb) != -1) {
+                int ret = lchown("freshclam.dat", user->pw_uid, user->pw_gid);
+                if (ret) {
+                    fprintf(stderr, "ERROR: lchown to user '%s' failed on freshclam.dat\n", user->pw_name);
+                    fprintf(stderr, "Error was '%s'\n", strerror(errno));
+                    logg("^lchown to user '%s' failed on freshclam.dat.  Error was '%s'\n",
+                         user->pw_name, strerror(errno));
+                    status = FC_ECONFIG;
+                    goto done;
+                }
+            }
+        }
+
         /*
          * freshclam shouldn't work with root privileges.
          * Drop privileges to the DatabaseOwner user, if specified.
@@ -1942,7 +1978,7 @@ int main(int argc, char **argv)
             alarm(0);
 #endif
 
-            if (ret > 1) {
+            if (ret > FC_UPTODATE) {
                 if ((opt = optget(opts, "OnErrorExecute"))->enabled)
                     arg = opt->strarg;
 
@@ -1950,6 +1986,14 @@ int main(int argc, char **argv)
                     execute("OnErrorExecute", arg, optget(opts, "daemon")->enabled);
 
                 arg = NULL;
+
+                if (FC_EFORBIDDEN == ret) {
+                    /* We're being actively blocked, which is a fatal error. Exit. */
+                    logg("^FreshClam was forbidden from downloading a database.\n");
+                    logg("^This is fatal. Retrying later won't help. Exiting now.\n");
+                    status = ret;
+                    goto done;
+                }
             }
 
             logg("#--------------------------------------\n");
