@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Nigel Horne
@@ -225,7 +225,7 @@ int blobAddData(blob *b, const unsigned char *data, size_t len)
 
         b->size = growth;
         b->data = cli_malloc(growth);
-        if (NULL == b->data){
+        if (NULL == b->data) {
             b->size = 0;
             return -1;
         }
@@ -245,7 +245,7 @@ int blobAddData(blob *b, const unsigned char *data, size_t len)
 
         b->size = (off_t)len * 4;
         b->data = cli_malloc(b->size);
-        if (NULL == b->data){
+        if (NULL == b->data) {
             b->size = 0;
             return -1;
         }
@@ -320,8 +320,9 @@ void blobClose(blob *b)
         } else {
             unsigned char *ptr = cli_realloc(b->data, b->len);
 
-            if (ptr == NULL)
+            if (ptr == NULL) {
                 return;
+            }
 
             cli_dbgmsg("blobClose: recovered %lu bytes from %lu\n",
                        (unsigned long)(b->size - b->len),
@@ -478,11 +479,11 @@ void fileblobDestroy(fileblob *fb)
     } else if (fb->b.data) {
         free(fb->b.data);
         if (fb->b.name) {
-            cli_errmsg("fileblobDestroy: %s not saved: report to https://bugzilla.clamav.net\n",
+            cli_errmsg("fileblobDestroy: %s not saved: report to https://github.com/Cisco-Talos/clamav/issues\n",
                        (fb->fullname) ? fb->fullname : fb->b.name);
             free(fb->b.name);
         } else
-            cli_errmsg("fileblobDestroy: file not saved (%lu bytes): report to https://bugzilla.clamav.net\n",
+            cli_errmsg("fileblobDestroy: file not saved (%lu bytes): report to https://github.com/Cisco-Talos/clamav/issues\n",
                        (unsigned long)fb->b.len);
     }
     if (fb->fullname)
@@ -516,7 +517,7 @@ void fileblobPartialSet(fileblob *fb, const char *fullname, const char *arg)
         close(fb->fd);
         return;
     }
-    blobSetFilename(&fb->b, fb->ctx ? fb->ctx->engine->tmpdir : NULL, fullname);
+    blobSetFilename(&fb->b, fb->ctx ? fb->ctx->sub_tmpdir : NULL, fullname);
     if (fb->b.data)
         if (fileblobAddData(fb, fb->b.data, fb->b.len) == 0) {
             free(fb->b.data);
@@ -595,7 +596,7 @@ int fileblobAddData(fileblob *fb, const unsigned char *data, size_t len)
                     *ctx->scanned += (unsigned long)len / CL_COUNT_PRECISION;
                 fb->bytes_scanned += (unsigned long)len;
 
-                if ((len > 5) && cli_updatelimits(ctx, len) == CL_CLEAN && (cli_scanbuff(data, (unsigned int)len, 0, ctx->virname, ctx->engine, CL_TYPE_BINARY_DATA, NULL) == CL_VIRUS)) {
+                if ((len > 5) && cli_updatelimits(ctx, len) == CL_CLEAN && (cli_scan_buff(data, (unsigned int)len, 0, ctx->virname, ctx->engine, CL_TYPE_BINARY_DATA, NULL) == CL_VIRUS)) {
                     cli_dbgmsg("fileblobAddData: found %s\n", cli_get_last_virus_str(ctx->virname));
                     fb->isInfected = 1;
                 }
@@ -660,7 +661,7 @@ int fileblobScan(const fileblob *fb)
         virus_found = 1;
     }
 
-    rc = cli_magic_scandesc(fb->fd, fb->fullname, fb->ctx);
+    rc = cli_magic_scan_desc(fb->fd, fb->fullname, fb->ctx, fb->b.name);
     if (rc == CL_VIRUS || virus_found != 0) {
         cli_dbgmsg("%s is infected\n", fb->fullname);
         return CL_VIRUS;
