@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -193,7 +193,7 @@ int cli_scanishield_msi(cli_ctx *ctx, off_t off)
     const uint8_t *buf;
     unsigned int fcount, scanned = 0;
     int ret;
-    fmap_t *map = *ctx->fmap;
+    fmap_t *map = ctx->fmap;
 
     cli_dbgmsg("in ishield-msi\n");
     if (!(buf = fmap_need_off_once(map, off, 0x20))) {
@@ -224,7 +224,7 @@ int cli_scanishield_msi(cli_ctx *ctx, off_t off)
         off += sizeof(fb);
         fb.fname[sizeof(fb.fname) - 1] = '\0';
         csize                          = le64_to_host(fb.csize);
-        if (!CLI_ISCONTAINED(0, map->len, off, csize)) {
+        if (!CLI_ISCONTAINED_0_TO(map->len, off, csize)) {
             cli_dbgmsg("ishield-msi: next stream is out of file, giving up\n");
             return CL_CLEAN;
         }
@@ -362,7 +362,7 @@ int cli_scanishield(cli_ctx *ctx, off_t off, size_t sz)
     long fsize;
     off_t coff           = off;
     struct IS_CABSTUFF c = {NULL, -1, 0, 0};
-    fmap_t *map          = *ctx->fmap;
+    fmap_t *map          = ctx->fmap;
     unsigned fc          = 0;
     int virus_found      = 0;
 
@@ -468,7 +468,7 @@ static int is_dump_and_scan(cli_ctx *ctx, off_t off, size_t fsize)
     char *fname;
     const char *buf;
     int ofd, ret = CL_CLEAN;
-    fmap_t *map = *ctx->fmap;
+    fmap_t *map = ctx->fmap;
 
     if (!fsize) {
         cli_dbgmsg("ishield: skipping empty file\n");
@@ -518,7 +518,7 @@ static int is_parse_hdr(cli_ctx *ctx, struct IS_CABSTUFF *c)
     unsigned int off, i, scanned = 0;
     int ret = CL_BREAK;
     char hash[33], *hdr;
-    fmap_t *map = *ctx->fmap;
+    fmap_t *map = ctx->fmap;
 
     const struct IS_HDR *h1;
     struct IS_OBJECTS *objs;
@@ -716,7 +716,7 @@ static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t cs
     z_stream z;
     uint64_t outsz = 0;
     int success    = 0;
-    fmap_t *map    = *ctx->fmap;
+    fmap_t *map    = ctx->fmap;
 
     if (!(outbuf = cli_malloc(IS_CABBUFSZ))) {
         cli_errmsg("is_extract_cab: Unable to allocate memory for outbuf\n");
@@ -773,7 +773,7 @@ static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t cs
             zret        = inflate(&z, 0);
             if (zret == Z_OK || zret == Z_STREAM_END || zret == Z_BUF_ERROR) {
                 unsigned int umpd = IS_CABBUFSZ - z.avail_out;
-                if (cli_writen(ofd, outbuf, umpd) < umpd)
+                if (cli_writen(ofd, outbuf, umpd) != umpd)
                     break;
                 outsz += umpd;
                 if (zret == Z_STREAM_END || z.avail_out == IS_CABBUFSZ /* FIXMEISHIELD: is the latter ok? */) {
