@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Kevin Lin <klin@sourcefire.com>
  *
@@ -64,17 +64,17 @@ enum MBR_STATE {
     SEEN_EMPTY
 };
 
-static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba,
+static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, size_t extlba,
                            size_t extlbasize, size_t sectorsize);
 static int mbr_check_mbr(struct mbr_boot_record *record, size_t maplen, size_t sectorsize);
 static int mbr_check_ebr(struct mbr_boot_record *record);
 static int mbr_primary_partition_intersection(cli_ctx *ctx, struct mbr_boot_record mbr, size_t sectorsize);
-static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t sectorsize);
+static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t sectorsize);
 
 int cli_mbr_check(const unsigned char *buff, size_t len, size_t maplen)
 {
     struct mbr_boot_record mbr;
-    off_t mbr_base    = 0;
+    size_t mbr_base   = 0;
     size_t sectorsize = 512;
 
     if (len < sectorsize) {
@@ -94,7 +94,7 @@ int cli_mbr_check(const unsigned char *buff, size_t len, size_t maplen)
 int cli_mbr_check2(cli_ctx *ctx, size_t sectorsize)
 {
     struct mbr_boot_record mbr;
-    off_t pos = 0, mbr_base = 0;
+    size_t pos = 0, mbr_base = 0;
     size_t maplen;
 
     if (!ctx || !ctx->fmap) {
@@ -109,7 +109,7 @@ int cli_mbr_check2(cli_ctx *ctx, size_t sectorsize)
     mbr_base = sectorsize - sizeof(struct mbr_boot_record);
 
     /* size of total file must be a multiple of the sector size */
-    maplen = (*ctx->fmap)->real_len;
+    maplen = ctx->fmap->len;
     if ((maplen % sectorsize) != 0) {
         cli_dbgmsg("cli_scanmbr: File sized %lu is not a multiple of sector size %lu\n",
                    (unsigned long)maplen, (unsigned long)sectorsize);
@@ -120,7 +120,7 @@ int cli_mbr_check2(cli_ctx *ctx, size_t sectorsize)
     pos = (MBR_SECTOR * sectorsize) + mbr_base;
 
     /* read the master boot record */
-    if (fmap_readn(*ctx->fmap, &mbr, pos, sizeof(mbr)) != sizeof(mbr)) {
+    if (fmap_readn(ctx->fmap, &mbr, pos, sizeof(mbr)) != sizeof(mbr)) {
         cli_dbgmsg("cli_scanmbr: Invalid master boot record\n");
         return CL_EFORMAT;
     }
@@ -140,7 +140,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     struct mbr_boot_record mbr;
     enum MBR_STATE state = SEEN_NOTHING;
     int ret = CL_CLEAN, detection = CL_CLEAN;
-    off_t pos = 0, mbr_base = 0, partoff = 0;
+    size_t pos = 0, mbr_base = 0, partoff = 0;
     unsigned i = 0, prtncount = 0;
     size_t maplen, partsize;
 
@@ -158,7 +158,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     mbr_base = sectorsize - sizeof(struct mbr_boot_record);
 
     /* size of total file must be a multiple of the sector size */
-    maplen = (*ctx->fmap)->real_len;
+    maplen = ctx->fmap->len;
     if ((maplen % sectorsize) != 0) {
         cli_dbgmsg("cli_scanmbr: File sized %lu is not a multiple of sector size %lu\n",
                    (unsigned long)maplen, (unsigned long)sectorsize);
@@ -169,7 +169,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     pos = (MBR_SECTOR * sectorsize) + mbr_base;
 
     /* read the master boot record */
-    if (fmap_readn(*ctx->fmap, &mbr, pos, sizeof(mbr)) != sizeof(mbr)) {
+    if (fmap_readn(ctx->fmap, &mbr, pos, sizeof(mbr)) != sizeof(mbr)) {
         cli_dbgmsg("cli_scanmbr: Invalid master boot record\n");
         return CL_EFORMAT;
     }
@@ -184,7 +184,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     }
 
     /* MBR is valid, examine bootstrap code */
-    ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, 0, sectorsize, ctx, CL_TYPE_ANY, NULL);
+    ret = cli_magic_scan_nested_fmap_type(ctx->fmap, 0, sectorsize, ctx, CL_TYPE_ANY, NULL);
     if (ret != CL_CLEAN) {
         if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
             detection = CL_VIRUS;
@@ -210,10 +210,10 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
         cli_dbgmsg("MBR Partition Entry %u:\n", i);
         cli_dbgmsg("Status: %u\n", mbr.entries[i].status);
         cli_dbgmsg("Type: %x\n", mbr.entries[i].type);
-        cli_dbgmsg("Blocks: [%u, +%u), ([%lu, +%lu))\n",
+        cli_dbgmsg("Blocks: [%u, +%u), ([%zu, +%zu))\n",
                    mbr.entries[i].firstLBA, mbr.entries[i].numLBA,
-                   (unsigned long)(mbr.entries[i].firstLBA * sectorsize),
-                   (unsigned long)(mbr.entries[i].numLBA * sectorsize));
+                   mbr.entries[i].firstLBA * sectorsize,
+                   mbr.entries[i].numLBA * sectorsize);
 
         /* Handle MBR entry based on type */
         if (mbr.entries[i].type == MBR_EMPTY) {
@@ -240,7 +240,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
             partoff  = mbr.entries[i].firstLBA * sectorsize;
             partsize = mbr.entries[i].numLBA * sectorsize;
             mbr_parsemsg("cli_magic_scan_nested_fmap_type: [%u, +%u)\n", partoff, partsize);
-            ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
+            ret = cli_magic_scan_nested_fmap_type(ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
             if (ret != CL_CLEAN) {
                 if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
                     detection = CL_VIRUS;
@@ -257,12 +257,12 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     return detection;
 }
 
-static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t extlbasize, size_t sectorsize)
+static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t extlbasize, size_t sectorsize)
 {
     struct mbr_boot_record ebr;
     enum MBR_STATE state = SEEN_NOTHING;
     int ret = CL_CLEAN, detection = CL_CLEAN;
-    off_t pos = 0, mbr_base = 0, logiclba = 0, extoff = 0, partoff = 0;
+    size_t pos = 0, mbr_base = 0, logiclba = 0, extoff = 0, partoff = 0;
     size_t partsize, extsize;
     unsigned i = 0, j = 0;
 
@@ -278,7 +278,7 @@ static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size
 
         /* read the extended boot record */
         pos += (logiclba * sectorsize) + mbr_base;
-        if (fmap_readn(*ctx->fmap, &ebr, pos, sizeof(ebr)) != sizeof(ebr)) {
+        if (fmap_readn(ctx->fmap, &ebr, pos, sizeof(ebr)) != sizeof(ebr)) {
             cli_dbgmsg("cli_scanebr: Invalid extended boot record\n");
             return CL_EFORMAT;
         }
@@ -381,7 +381,7 @@ static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size
                         return CL_EFORMAT;
                     }
 
-                    ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
+                    ret = cli_magic_scan_nested_fmap_type(ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
                     if (ret != CL_CLEAN) {
                         if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
                             detection = CL_VIRUS;
@@ -424,7 +424,7 @@ void mbr_convert_to_host(struct mbr_boot_record *record)
 static int mbr_check_mbr(struct mbr_boot_record *record, size_t maplen, size_t sectorsize)
 {
     unsigned i      = 0;
-    off_t partoff   = 0;
+    size_t partoff  = 0;
     size_t partsize = 0;
 
     for (i = 0; i < MBR_MAX_PARTITION_ENTRIES; ++i) {
@@ -540,13 +540,13 @@ leave:
 }
 
 /* checks internal logical partitions */
-static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t sectorsize)
+static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t sectorsize)
 {
     struct mbr_boot_record ebr;
     partition_intersection_list_t prtncheck;
     unsigned i, pitxn;
     int ret = CL_CLEAN, tmp = CL_CLEAN, mbr_base = 0;
-    off_t pos = 0, logiclba = 0;
+    size_t pos = 0, logiclba = 0;
     int virus_found = 0;
 
     mbr_base = sectorsize - sizeof(struct mbr_boot_record);
@@ -560,7 +560,7 @@ static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount
 
         /* read the extended boot record */
         pos += (logiclba * sectorsize) + mbr_base;
-        if (fmap_readn(*ctx->fmap, &ebr, pos, sizeof(ebr)) != sizeof(ebr)) {
+        if (fmap_readn(ctx->fmap, &ebr, pos, sizeof(ebr)) != sizeof(ebr)) {
             cli_dbgmsg("cli_scanebr: Invalid extended boot record\n");
             partition_intersection_list_free(&prtncheck);
             return CL_EFORMAT;
