@@ -562,10 +562,15 @@ ole2_walk_property_tree(ole2_header_t *hdr, const char *dir, int32_t prop_index,
     if ((rec_level > 100) || (*file_count > 100000)) {
         return CL_SUCCESS;
     }
-    if (ctx && ctx->engine->maxreclevel && (rec_level > ctx->engine->maxreclevel)) {
-        cli_dbgmsg("OLE2: Recursion limit reached (max: %d)\n", ctx->engine->maxreclevel);
-        return CL_SUCCESS;
+
+    if (ctx && ctx->engine->max_recursion_level && (rec_level > ctx->engine->max_recursion_level)) {
+        // Note: engine->max_recursion_level is re-purposed here out of convenience.
+        //       ole2 recursion does not leverage the ctx->recursion_stack stack.
+        cli_dbgmsg("OLE2: Recursion limit reached (max: %d)\n", ctx->engine->max_recursion_level);
+        cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxRecursion");
+        return CL_EMAXREC;
     }
+
     //push the 'root' node for the level onto the local list
     if ((ret = ole2_list_push(&node_list, prop_index)) != CL_SUCCESS) {
         ole2_list_delete(&node_list);
@@ -673,8 +678,9 @@ ole2_walk_property_tree(ole2_header_t *hdr, const char *dir, int32_t prop_index,
                 break;
             case 2: /* File */
                 ole2_listmsg("file node\n");
-                if (ctx && ctx->engine->maxfiles && ctx->scannedfiles + *file_count > ctx->engine->maxfiles) {
+                if (ctx && ctx->engine->maxfiles && ((*file_count > ctx->engine->maxfiles) || (ctx->scannedfiles > ctx->engine->maxfiles - *file_count))) {
                     cli_dbgmsg("OLE2: files limit reached (max: %u)\n", ctx->engine->maxfiles);
+                    cli_append_virus_if_heur_exceedsmax(ctx, "Heuristics.Limits.Exceeded.MaxFiles");
                     ole2_list_delete(&node_list);
                     return CL_EMAXFILES;
                 }
@@ -991,8 +997,8 @@ scan_biff_for_xlm_macros(struct biff_parser_state *state, unsigned char *buff, s
                 break;
             default:
                 switch (state->state) {
-#if HAVE_JSON
                     case BIFF_PARSER_NAME_RECORD:
+#if HAVE_JSON
                         if (state->data_offset == 0) {
                             state->tmp = buff[i] & 0x20;
                         } else if ((state->data_offset == 14 || state->data_offset == 15) && state->tmp) {
@@ -1011,8 +1017,8 @@ scan_biff_for_xlm_macros(struct biff_parser_state *state, unsigned char *buff, s
                                 state->tmp = 0;
                             }
                         }
-                        break;
 #endif
+                        break;
                     case BIFF_PARSER_BOUNDSHEET_RECORD:
                         if (state->data_offset == 4) {
                             state->tmp = buff[i];
@@ -1044,7 +1050,7 @@ scan_biff_for_xlm_macros(struct biff_parser_state *state, unsigned char *buff, s
                         break;
                     default:
                         //Should never arrive here
-                        cli_errmsg("[scan_biff_for_xlm_macros] Unexpected state value %d\n", (int)state->state);
+                        cli_dbgmsg("[scan_biff_for_xlm_macros] Unexpected state value %d\n", (int)state->state);
                         break;
                 }
                 state->data_offset += 1;
@@ -1738,10 +1744,10 @@ int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **files, int
                sizeof(off_t) - sizeof(bitset_t *) -
                sizeof(struct uniq *) - sizeof(fmap_t *) - sizeof(int) - sizeof(hwp5_header_t *);
 
-    if ((size_t)((*ctx->fmap)->len) < (size_t)(hdr_size)) {
+    if ((size_t)(ctx->fmap->len) < (size_t)(hdr_size)) {
         return CL_CLEAN;
     }
-    hdr.map      = *ctx->fmap;
+    hdr.map      = ctx->fmap;
     hdr.m_length = hdr.map->len;
     phdr         = fmap_need_off_once(hdr.map, 0, hdr_size);
     if (phdr) {
