@@ -114,6 +114,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
     int err                    = Z_OK;
     uint8_t *decompressed_data = NULL;
 
+    bool zstrm_initialized = false;
     z_stream zstrm;
     size_t decompressed_data_len = 0;
 
@@ -126,7 +127,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
         status = CL_EARG;
         goto done;
     }
-    map = *ctx->fmap;
+    map = ctx->fmap;
 
     while (fmap_readn(map, (void *)&chunk_data_length_u32, offset, PNG_CHUNK_LENGTH_SIZE) == PNG_CHUNK_LENGTH_SIZE) {
         chunk_data_length = be32_to_host(chunk_data_length_u32);
@@ -297,6 +298,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
 
                     idat_state = PNG_IDAT_DECOMPRESSION_FAILED;
                 } else {
+                    zstrm_initialized = true;
                     uint64_t cur_width, cur_linebytes;
                     int64_t cur_xoff  = 0;
                     int64_t cur_xskip = interlace_method ? 8 : 1;
@@ -343,6 +345,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
                     if (err != Z_OK && err != Z_STREAM_END) {
                         cli_dbgmsg("PNG: zlib: inflate error: %d, Image decompression failed!\n", err);
                         inflateEnd(&zstrm);
+                        zstrm_initialized = false;
                         idat_state = PNG_IDAT_DECOMPRESSION_FAILED;
                         break;
                     }
@@ -351,9 +354,10 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
                 if (err == Z_STREAM_END) {
                     cli_dbgmsg("  TOTAL decompressed:    %zu\n", decompressed_data_len);
                     inflateEnd(&zstrm);
+                    zstrm_initialized = false;
                     idat_state = PNG_IDAT_DECOMPRESSION_COMPLETE;
 
-                    if (decompressed_data_len > image_size) {
+                    if ((decompressed_data_len > image_size) && (SCAN_HEURISTIC_BROKEN_MEDIA)) {
                         status = cli_append_virus(ctx, "Heuristics.PNG.CVE-2010-1205");
                         goto done;
                     }
@@ -436,7 +440,7 @@ done:
     if (NULL != decompressed_data) {
         free(decompressed_data);
     }
-    if (idat_state == PNG_IDAT_DECOMPRESSION_IN_PROGRESS) {
+    if (zstrm_initialized) {
         inflateEnd(&zstrm);
     }
 
