@@ -113,6 +113,10 @@ const char *fc_strerror(fc_error_t fcerror)
             return "Memory allocation error";
         case FC_EARG:
             return "Invalid argument(s)";
+        case FC_EFORBIDDEN:
+            return "Forbidden; Blocked by CDN";
+        case FC_ERETRYLATER:
+            return "Too many requests; Retry later";
         default:
             return "Unknown libfreshclam error code!";
     }
@@ -246,6 +250,17 @@ fc_error_t fc_initialize(fc_config *fcConfig)
 
     g_bCompressLocalDatabase = fcConfig->bCompressLocalDatabase;
 
+    /* Load or create freshclam.dat */
+    if (FC_SUCCESS != load_freshclam_dat()) {
+        logg("*Failed to load freshclam.dat; will create a new freshclam.dat\n");
+
+        if (FC_SUCCESS != new_freshclam_dat()) {
+            logg("^Failed to create a new freshclam.dat!\n");
+            status = FC_EINIT;
+            goto done;
+        }
+    }
+
     status = FC_SUCCESS;
 
 done:
@@ -293,6 +308,10 @@ void fc_cleanup(void)
         free(g_tempDirectory);
         g_tempDirectory = NULL;
     }
+    if (NULL != g_freshclamDat) {
+        free(g_freshclamDat);
+        g_freshclamDat = NULL;
+    }
 }
 
 fc_error_t fc_prune_database_directory(char **databaseList, uint32_t nDatabases)
@@ -542,7 +561,7 @@ fc_error_t fc_dns_query_update_info(
 
                 logg("^Your ClamAV installation is OUTDATED!\n");
                 logg("^Local version: %s Recommended version: %s\n", version_string, reply_token);
-                logg("DON'T PANIC! Read https://www.clamav.net/documents/upgrading-clamav\n");
+                logg("DON'T PANIC! Read https://docs.clamav.net/manual/Installing.html\n");
                 *newVersion = cli_strdup(reply_token);
             }
         }
@@ -627,8 +646,7 @@ fc_error_t fc_update_database(
                 }
                 case FC_ECONNECTION:
                 case FC_EBADCVD:
-                case FC_EFAILEDGET:
-                case FC_EMIRRORNOTSYNC: {
+                case FC_EFAILEDGET: {
                     if (attempt < g_maxAttempts) {
                         logg("Trying again in 5 secs...\n");
                         sleep(5);
@@ -642,8 +660,62 @@ fc_error_t fc_update_database(
                     }
                     break;
                 }
+                case FC_EMIRRORNOTSYNC: {
+                    logg("Received an older %s CVD than was advertised. We'll retry so the incremental update will ensure we're up-to-date.\n", database);
+                    break;
+                }
+                case FC_EFORBIDDEN: {
+                    char retry_after_string[26];
+                    struct tm *tm_info;
+                    tm_info = localtime(&g_freshclamDat->retry_after);
+                    if (NULL == tm_info) {
+                        logg("!Failed to query the local time for the retry-after date!\n");
+                        status = FC_ERROR;
+                        goto done;
+                    }
+                    strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                    logg("^FreshClam received error code 403 from the ClamAV Content Delivery Network (CDN).\n");
+                    logg("This could mean several things:\n");
+                    logg(" 1. You are running an out-of-date version of ClamAV / FreshClam.\n");
+                    logg("    Ensure you are the most updated version by visiting https://www.clamav.net/downloads\n");
+                    logg(" 2. Your network is explicitly denied by the FreshClam CDN.\n");
+                    logg("    In order to rectify this please check that you are:\n");
+                    logg("   a. Running an up-to-date version of FreshClam\n");
+                    logg("   b. Running FreshClam no more than once an hour\n");
+                    logg("   c. If you have checked (a) and (b), please open a ticket at\n");
+                    logg("      https://github.com/Cisco-Talos/clamav/issues\n");
+                    logg("      and we will investigate why your network is blocked.\n");
+                    logg("^You are on cool-down until after: %s\n", retry_after_string);
+                    status = ret;
+                    goto done;
+                    break;
+                }
+                case FC_ERETRYLATER: {
+                    char retry_after_string[26];
+                    struct tm *tm_info;
+                    tm_info = localtime(&g_freshclamDat->retry_after);
+                    if (NULL == tm_info) {
+                        logg("!Failed to query the local time for the retry-after date!\n");
+                        status = FC_ERROR;
+                        goto done;
+                    }
+                    strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                    logg("^FreshClam received error code 429 from the ClamAV Content Delivery Network (CDN).\n");
+                    logg("This means that you have been rate limited by the CDN.\n");
+                    logg(" 1. Run FreshClam no more than once an hour to check for updates.\n");
+                    logg("    FreshClam should check DNS first to see if an update is needed.\n");
+                    logg(" 2. If you have more than 10 hosts on your network attempting to download,\n");
+                    logg("    it is recommended that you set up a private mirror on your network using\n");
+                    logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+                    logg("    CDN and your own network.\n");
+                    logg(" 3. Please do not open a ticket asking for an exemption from the rate limit,\n");
+                    logg("    it will not be granted.\n");
+                    logg("^You are on cool-down until after: %s\n", retry_after_string);
+                    goto success;
+                    break;
+                }
                 default: {
-                    logg("!Unexpected error when attempting to update database: %s\n", database);
+                    logg("!Unexpected error when attempting to update %s: %s\n", database, fc_strerror(ret));
                     status = ret;
                     goto done;
                 }
@@ -688,6 +760,40 @@ fc_error_t fc_update_databases(
 
     *nUpdated = 0;
 
+    if (g_freshclamDat->retry_after > 0) {
+        if (g_freshclamDat->retry_after > time(NULL)) {
+            /* We're on cool-down, try again later. */
+            char retry_after_string[26];
+            struct tm *tm_info;
+            tm_info = localtime(&g_freshclamDat->retry_after);
+            if (NULL == tm_info) {
+                logg("!Failed to query the local time for the retry-after date!\n");
+                status = FC_ERROR;
+                goto done;
+            }
+            strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+            logg("^FreshClam previously received error code 429 or 403 from the ClamAV Content Delivery Network (CDN).\n");
+            logg("This means that you have been rate limited or blocked by the CDN.\n");
+            logg(" 1. Verify that you're running a supported ClamAV version.\n");
+            logg("    See https://docs.clamav.net/faq/faq-eol.html for details.\n");
+            logg(" 2. Run FreshClam no more than once an hour to check for updates.\n");
+            logg("    FreshClam should check DNS first to see if an update is needed.\n");
+            logg(" 3. If you have more than 10 hosts on your network attempting to download,\n");
+            logg("    it is recommended that you set up a private mirror on your network using\n");
+            logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+            logg("    CDN and your own network.\n");
+            logg(" 4. Please do not open a ticket asking for an exemption from the rate limit,\n");
+            logg("    it will not be granted.\n");
+            logg("^You are still on cool-down until after: %s\n", retry_after_string);
+            status = FC_SUCCESS;
+            goto done;
+        } else {
+            g_freshclamDat->retry_after = 0;
+            logg("^Cool-down expired, ok to try again.\n");
+            save_freshclam_dat();
+        }
+    }
+
     for (i = 0; i < nDatabases; i++) {
         if (FC_SUCCESS != (ret = fc_update_database(
                                databaseList[i],
@@ -698,7 +804,6 @@ fc_error_t fc_update_databases(
                                bScriptedUpdates,
                                context,
                                &bUpdated))) {
-            logg("^fc_update_databases: fc_update_database failed: %s (%d)\n", fc_strerror(ret), ret);
             status = ret;
             goto done;
         }
@@ -777,6 +882,56 @@ fc_error_t fc_download_url_database(
                 }
                 break;
             }
+            case FC_EFORBIDDEN: {
+                char retry_after_string[26];
+                struct tm *tm_info;
+                tm_info = localtime(&g_freshclamDat->retry_after);
+                if (NULL == tm_info) {
+                    logg("!Failed to query the local time for the retry-after date!\n");
+                    status = FC_ERROR;
+                    goto done;
+                }
+                strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                logg("^FreshClam received error code 403 from the ClamAV Content Delivery Network (CDN).\n");
+                logg("This could mean several things:\n");
+                logg(" 1. You are running an out-of-date version of ClamAV / FreshClam.\n");
+                logg("    Ensure you are the most updated version by visiting https://www.clamav.net/downloads\n");
+                logg(" 2. Your network is explicitly denied by the FreshClam CDN.\n");
+                logg("    In order to rectify this please check that you are:\n");
+                logg("   a. Running an up-to-date version of FreshClam\n");
+                logg("   b. Running FreshClam no more than once an hour\n");
+                logg("   c. If you have checked (a) and (b), please open a ticket at\n");
+                logg("      https://github.com/Cisco-Talos/clamav/issues\n");
+                logg("      and we will investigate why your network is blocked.\n");
+                logg("^You are on cool-down until after: %s\n", retry_after_string);
+                status = ret;
+                goto done;
+                break;
+            }
+            case FC_ERETRYLATER: {
+                char retry_after_string[26];
+                struct tm *tm_info;
+                tm_info = localtime(&g_freshclamDat->retry_after);
+                if (NULL == tm_info) {
+                    logg("!Failed to query the local time for the retry-after date!\n");
+                    status = FC_ERROR;
+                    goto done;
+                }
+                strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                logg("^FreshClam received error code 429 from the ClamAV Content Delivery Network (CDN).\n");
+                logg("This means that you have been rate limited by the CDN.\n");
+                logg(" 1. Run FreshClam no more than once an hour to check for updates.\n");
+                logg("    FreshClam should check DNS first to see if an update is needed.\n");
+                logg(" 2. If you have more than 10 hosts on your network attempting to download,\n");
+                logg("    it is recommended that you set up a private mirror on your network using\n");
+                logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+                logg("    CDN and your own network.\n");
+                logg(" 3. Please do not open a ticket asking for an exemption from the rate limit,\n");
+                logg("    it will not be granted.\n");
+                logg("^You are on cool-down until after: %s\n", retry_after_string);
+                goto success;
+                break;
+            }
             default: {
                 logg("Unexpected error when attempting to update from custom database URL: %s\n", urlDatabase);
                 status = ret;
