@@ -66,6 +66,7 @@
 #include <math.h>
 
 #include <curl/curl.h>
+#include <openssl/rand.h>
 
 #include "target.h"
 
@@ -115,6 +116,263 @@ uint32_t g_requestTimeout = 0;
 
 uint32_t g_bCompressLocalDatabase = 0;
 
+freshclam_dat_v1_t *g_freshclamDat = NULL;
+
+/** @brief Generate a Version 4 UUID according to RFC-4122
+ *
+ * Uses the openssl RAND_bytes function to generate a Version 4 UUID.
+ *
+ * Copyright 2021 Karthik Velakur with some modifications by the ClamAV team.
+ * License: MIT
+ * From: https://gist.github.com/kvelakur/9069c9896577c3040030
+ *
+ * @param buffer A buffer that is SIZEOF_UUID_V4
+ */
+static void uuid_v4_gen(char *buffer)
+{
+    union {
+        struct
+        {
+            uint32_t time_low;
+            uint16_t time_mid;
+            uint16_t time_hi_and_version;
+            uint8_t clk_seq_hi_res;
+            uint8_t clk_seq_low;
+            uint8_t node[6];
+        };
+        uint8_t __rnd[16];
+    } uuid;
+
+    if (0 >= RAND_bytes(uuid.__rnd, sizeof(uuid.__rnd))) {
+        /* Failed to generate random bytes for new UUID */
+        memset(uuid.__rnd, 0, sizeof(uuid.__rnd));
+        uuid.time_low = (uint32_t)time(NULL);
+    }
+
+    // Refer Section 4.2 of RFC-4122
+    // https://tools.ietf.org/html/rfc4122#section-4.2
+    uuid.clk_seq_hi_res      = (uint8_t)((uuid.clk_seq_hi_res & 0x3F) | 0x80);
+    uuid.time_hi_and_version = (uint16_t)((uuid.time_hi_and_version & 0x0FFF) | 0x4000);
+
+    snprintf(buffer, SIZEOF_UUID_V4, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
+             uuid.time_low, uuid.time_mid, uuid.time_hi_and_version,
+             uuid.clk_seq_hi_res, uuid.clk_seq_low,
+             uuid.node[0], uuid.node[1], uuid.node[2],
+             uuid.node[3], uuid.node[4], uuid.node[5]);
+    buffer[SIZEOF_UUID_V4 - 1] = 0;
+
+    return;
+}
+
+fc_error_t load_freshclam_dat(void)
+{
+    fc_error_t status      = FC_EINIT;
+    int handle             = -1;
+    ssize_t bread          = 0;
+    freshclam_dat_v1_t *mdat = NULL;
+    uint32_t version       = 0;
+    char magic[13]         = {0};
+
+    /* Change directory to database directory */
+    if (chdir(g_databaseDirectory)) {
+        logg("!Can't change dir to %s\n", g_databaseDirectory);
+        status = FC_EDIRECTORY;
+        goto done;
+    }
+    logg("*Current working dir is %s\n", g_databaseDirectory);
+
+    if (-1 == (handle = open("freshclam.dat", O_RDONLY | O_BINARY))) {
+        char currdir[PATH_MAX];
+
+        if (getcwd(currdir, sizeof(currdir)))
+            logg("*Can't open freshclam.dat in %s\n", currdir);
+        else
+            logg("*Can't open freshclam.dat in the current directory\n");
+
+        logg("*It probably doesn't exist yet. That's ok.\n");
+        status = FC_EFILE;
+        goto done;
+    }
+
+    if (strlen(MIRRORS_DAT_MAGIC) != (bread = read(handle, &magic, strlen(MIRRORS_DAT_MAGIC)))) {
+        char error_message[260];
+        cli_strerror(errno, error_message, 260);
+        logg("!Can't read magic from freshclam.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+        goto done;
+    }
+    if (0 != strncmp(magic, MIRRORS_DAT_MAGIC, strlen(MIRRORS_DAT_MAGIC))) {
+        logg("*Magic bytes for freshclam.dat did not match expectations.\n");
+        goto done;
+    }
+
+    if (sizeof(uint32_t) != (bread = read(handle, &version, sizeof(uint32_t)))) {
+        char error_message[260];
+        cli_strerror(errno, error_message, 260);
+        logg("!Can't read version from freshclam.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+        goto done;
+    }
+
+    switch (version) {
+        case 1: {
+            /* Verify that file size is as expected. */
+            off_t file_size = lseek(handle, 0L, SEEK_END);
+
+            if (strlen(MIRRORS_DAT_MAGIC) + sizeof(freshclam_dat_v1_t) != (size_t)file_size) {
+                logg("*freshclam.dat is bigger than expected: %zu != %ld\n", sizeof(freshclam_dat_v1_t), file_size);
+                goto done;
+            }
+
+            /* Rewind to just after the magic bytes and read data struct */
+            lseek(handle, strlen(MIRRORS_DAT_MAGIC), SEEK_SET);
+
+            mdat = malloc(sizeof(freshclam_dat_v1_t));
+            if (NULL == mdat) {
+                logg("!Failed to allocate memory for freshclam.dat\n");
+                status = FC_EMEM;
+                goto done;
+            }
+
+            if (sizeof(freshclam_dat_v1_t) != (bread = read(handle, mdat, sizeof(freshclam_dat_v1_t)))) {
+                char error_message[260];
+                cli_strerror(errno, error_message, 260);
+                logg("!Can't read from freshclam.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+                goto done;
+            }
+
+            /* Got it. */
+            close(handle);
+            handle = -1;
+
+            /* This is the latest version.
+               If we change the format in the future, we may wish to create a new
+               freshclam.dat struct, import the relevant bits to the new format,
+               and then save (overwrite) freshclam.dat with the new data. */
+            if (NULL != g_freshclamDat) {
+                free(g_freshclamDat);
+            }
+            g_freshclamDat = mdat;
+            mdat         = NULL;
+            break;
+        }
+        default: {
+            logg("*freshclam.dat version is different than expected: %u != %u\n", 1, version);
+            goto done;
+        }
+    }
+
+    logg("*Loaded freshclam.dat:\n");
+    logg("*  version:    %d\n", g_freshclamDat->version);
+    logg("*  uuid:       %s\n", g_freshclamDat->uuid);
+    if (g_freshclamDat->retry_after > 0) {
+        char retry_after_string[26];
+        struct tm *tm_info = localtime(&g_freshclamDat->retry_after);
+        if (NULL == tm_info) {
+            logg("!Failed to query the local time for the retry-after date!\n");
+            goto done;
+        }
+        strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+        logg("*  retry-after: %s\n", retry_after_string);
+    }
+
+    status = FC_SUCCESS;
+
+done:
+    if (-1 != handle) {
+        close(handle);
+    }
+    if (FC_SUCCESS != status) {
+        if (NULL != mdat) {
+            free(mdat);
+        }
+        if (NULL != g_freshclamDat) {
+            free(g_freshclamDat);
+            g_freshclamDat = NULL;
+        }
+    }
+
+    return status;
+}
+
+fc_error_t save_freshclam_dat(void)
+{
+    fc_error_t status = FC_EINIT;
+    int handle        = -1;
+
+    if (NULL == g_freshclamDat) {
+        logg("!Attempted to save freshclam.dat before initializing data struct!\n");
+        goto done;
+    }
+
+    if (-1 == (handle = open("freshclam.dat", O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644))) {
+        char currdir[PATH_MAX];
+
+        if (getcwd(currdir, sizeof(currdir)))
+            logg("!Can't create freshclam.dat in %s\n", currdir);
+        else
+            logg("!Can't create freshclam.dat in the current directory\n");
+
+        logg("Hint: The database directory must be writable for UID %d or GID %d\n", getuid(), getgid());
+        status = FC_EDBDIRACCESS;
+        goto done;
+    }
+    if (-1 == write(handle, MIRRORS_DAT_MAGIC, strlen(MIRRORS_DAT_MAGIC))) {
+        logg("!Can't write to freshclam.dat\n");
+    }
+    if (-1 == write(handle, g_freshclamDat, sizeof(freshclam_dat_v1_t))) {
+        logg("!Can't write to freshclam.dat\n");
+    }
+
+    logg("*Saved freshclam.dat\n");
+
+    status = FC_SUCCESS;
+done:
+    if (-1 != handle) {
+        close(handle);
+    }
+
+    return status;
+}
+
+fc_error_t new_freshclam_dat(void)
+{
+    fc_error_t status = FC_EINIT;
+
+    freshclam_dat_v1_t *mdat = calloc(1, sizeof(freshclam_dat_v1_t));
+    if (NULL == mdat) {
+        logg("!Failed to allocate memory for freshclam.dat\n");
+        status = FC_EMEM;
+        goto done;
+    }
+
+    mdat->version     = 1;
+    mdat->retry_after = 0;
+    uuid_v4_gen(mdat->uuid);
+
+    if (NULL != g_freshclamDat) {
+        free(g_freshclamDat);
+    }
+    g_freshclamDat = mdat;
+
+    logg("*Creating new freshclam.dat\n");
+
+    if (FC_SUCCESS != save_freshclam_dat()) {
+        logg("!Failed to save freshclam.dat!\n");
+        status = FC_EFILE;
+        goto done;
+    }
+
+    status = FC_SUCCESS;
+
+done:
+    if (FC_SUCCESS != status) {
+        if (NULL != mdat) {
+            free(mdat);
+        }
+        g_freshclamDat = NULL;
+    }
+    return status;
+}
+
 /**
  * @brief Get DNS text record field # for official databases.
  *
@@ -135,7 +393,7 @@ static int textrecordfield(const char *database)
     return 0;
 }
 
-#if LIBCURL_VERSION_NUM >= 0x073d00
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 61))
 /* In libcurl 7.61.0, support was added for extracting the time in plain
    microseconds. Older libcurl versions are stuck in using 'double' for this
    information so we complicate this example a bit by supporting either
@@ -282,7 +540,7 @@ static int xferinfo(void *prog,
     return 0;
 }
 
-#if LIBCURL_VERSION_NUM < 0x072000
+#if (LIBCURL_VERSION_MAJOR < 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR < 32))
 /**
  * Function from curl example code, Copyright (C) 1998 - 2018, Daniel Stenberg, see COPYING.curl for license details
  * Older style progress bar callback shim; for libcurl older than 7.32.0 ( CURLOPT_PROGRESSFUNCTION ).
@@ -328,12 +586,19 @@ static fc_error_t create_curl_handle(
         goto done;
     }
 
-    if (g_userAgent)
+    if (g_userAgent) {
         strncpy(userAgent, g_userAgent, sizeof(userAgent));
-    else
+    } else {
+        /*
+         * Use a randomly generated UUID in the User-Agent
+         * We'll try to load it from a file in the database directory.
+         * If none exists, we'll create a new one and save it to said file.
+         */
         snprintf(userAgent, sizeof(userAgent),
-                 PACKAGE "/%s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ")",
-                 get_version());
+                 PACKAGE "/%s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ", UUID: %s)",
+                 get_version(),
+                 g_freshclamDat->uuid);
+    }
     userAgent[sizeof(userAgent) - 1] = 0;
 
     if (mprintf_verbose) {
@@ -575,13 +840,11 @@ static fc_error_t remote_cvdhead(
     /*
      * Request CVD header.
      */
-    logg("Reading CVD header (%s): ", cvdfile);
-
     urlLen = strlen(server) + strlen("/") + strlen(cvdfile);
     url    = malloc(urlLen + 1);
     snprintf(url, urlLen + 1, "%s/%s", server, cvdfile);
 
-    logg("*Trying to retrieve CVD header from %s\n", url);
+    logg("Trying to retrieve CVD header from %s\n", url);
 
     if (FC_SUCCESS != (ret = create_curl_handle(
                            bHttpServer, // Set extra HTTP-specific headers.
@@ -602,7 +865,7 @@ static fc_error_t remote_cvdhead(
         prog.curl        = curl;
         prog.bComplete   = 0;
 
-#if LIBCURL_VERSION_NUM >= 0x072000
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 32))
         /* xferinfo was introduced in 7.32.0, no earlier libcurl versions will
        compile as they won't have the symbols around.
 
@@ -732,6 +995,40 @@ static fc_error_t remote_cvdhead(
             status = FC_UPTODATE;
             goto done;
         }
+        case 403: {
+            status = FC_EFORBIDDEN;
+
+            /* Try again in no less than 24 hours if freshclam received a 403 FORBIDDEN. */
+            g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 24;
+
+            (void)save_freshclam_dat();
+
+            break;
+        }
+        case 429: {
+            status = FC_ERETRYLATER;
+
+            curl_off_t retry_after = 0;
+
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 66))
+            /* CURLINFO_RETRY_AFTER was introduced in libcurl 7.66 */
+
+            /* Find out how long we should wait before allowing a retry. */
+            curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
+#endif
+
+            if (retry_after > 0) {
+                /* The response gave us a Retry-After date. Use that. */
+                g_freshclamDat->retry_after = time(NULL) + (time_t)retry_after;
+            } else {
+                /* Try again in no less than 4 hours if the response didn't specify
+                   or if CURLINFO_RETRY_AFTER is not supported. */
+                g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 4;
+            }
+            (void)save_freshclam_dat();
+
+            break;
+        }
         case 404: {
             if (g_proxyServer)
                 logg("^remote_cvdhead: file not found: %s (Proxy: %s:%u)\n", url, g_proxyServer, g_proxyPort);
@@ -865,7 +1162,7 @@ static fc_error_t downloadFile(
         prog.curl        = curl;
         prog.bComplete   = 0;
 
-#if LIBCURL_VERSION_NUM >= 0x072000
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 32))
         /* xferinfo was introduced in 7.32.0, no earlier libcurl versions will
        compile as they won't have the symbols around.
 
@@ -993,13 +1290,51 @@ static fc_error_t downloadFile(
     switch (http_code) {
         case 200:
         case 206: {
-            status = FC_SUCCESS;
+            if (0 == receivedFile.size) {
+                status = FC_EEMPTYFILE;
+            } else {
+                status = FC_SUCCESS;
+            }
             break;
         }
         case 304: {
             status = FC_UPTODATE;
             break;
         }
+        case 403: {
+            status = FC_EFORBIDDEN;
+
+            /* Try again in no less than 24 hours if freshclam received a 403 FORBIDDEN. */
+            g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 24;
+
+            (void)save_freshclam_dat();
+
+            break;
+        }
+        case 429: {
+            status = FC_ERETRYLATER;
+
+            curl_off_t retry_after = 0;
+
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 66))
+            /* CURLINFO_RETRY_AFTER was introduced in libcurl 7.66 */
+
+            /* Find out how long we should wait before allowing a retry. */
+            curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
+#endif
+
+            if (retry_after > 0) {
+                /* The response gave us a Retry-After date. Use that. */
+                g_freshclamDat->retry_after = time(NULL) + (time_t)retry_after;
+            } else {
+                /* Try again in no less than 4 hours if the response didn't specify
+                   or if CURLINFO_RETRY_AFTER is not supported. */
+                g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 4;
+            }
+            (void)save_freshclam_dat();
+
+            break;
+        }
         case 404: {
             if (g_proxyServer)
                 logg("^downloadFile: file not found: %s (Proxy: %s:%u)\n", url, g_proxyServer, g_proxyPort);
@@ -1050,6 +1385,7 @@ static fc_error_t getcvd(
     const char *cvdfile,
     const char *tmpfile,
     char *server,
+    uint32_t ifModifiedSince,
     unsigned int remoteVersion,
     int logerr)
 {
@@ -1071,8 +1407,13 @@ static fc_error_t getcvd(
     url    = malloc(urlLen + 1);
     snprintf(url, urlLen + 1, "%s/%s", server, cvdfile);
 
-    if (FC_SUCCESS != (ret = downloadFile(url, tmpfile, 1, logerr, 0))) {
-        logg("%cgetcvd: Can't download %s from %s\n", logerr ? '!' : '^', cvdfile, url);
+    ret = downloadFile(url, tmpfile, 1, logerr, ifModifiedSince);
+    if (ret == FC_UPTODATE) {
+        logg("%s is up-to-date.\n", cvdfile);
+        status = ret;
+        goto done;
+    } else if (ret > FC_UPTODATE) {
+        logg("%cCan't download %s from %s\n", logerr ? '!' : '^', cvdfile, url);
         status = ret;
         goto done;
     }
@@ -1080,55 +1421,42 @@ static fc_error_t getcvd(
     /* Temporarily rename file to correct extension for verification. */
     tmpfile_with_extension = strdup(tmpfile);
     if (!tmpfile_with_extension) {
-        logg("!getcvd: Can't allocate memory for temp file with extension!\n");
+        logg("!Can't allocate memory for temp file with extension!\n");
         status = FC_EMEM;
         goto done;
     }
     strncpy(tmpfile_with_extension + strlen(tmpfile_with_extension) - 4, cvdfile + strlen(cvdfile) - 4, 4);
     if (rename(tmpfile, tmpfile_with_extension) == -1) {
-        logg("!getcvd: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+        logg("!Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
         status = FC_EDBDIRACCESS;
         goto done;
     }
 
     if (CL_SUCCESS != (cl_ret = cl_cvdverify(tmpfile_with_extension))) {
-        logg("!getcvd: Verification: %s\n", cl_strerror(cl_ret));
+        logg("!Verification: %s\n", cl_strerror(cl_ret));
         status = FC_EBADCVD;
         goto done;
     }
 
     if (NULL == (cvd = cl_cvdhead(tmpfile_with_extension))) {
-        logg("!getcvd: Can't read CVD header of new %s database.\n", cvdfile);
+        logg("!Can't read CVD header of new %s database.\n", cvdfile);
         status = FC_EBADCVD;
         goto done;
     }
 
     /* Rename the file back to the original, since verification passed. */
     if (rename(tmpfile_with_extension, tmpfile) == -1) {
-        logg("!getcvd: Can't rename %s to %s: %s\n", tmpfile_with_extension, tmpfile, strerror(errno));
+        logg("!Can't rename %s to %s: %s\n", tmpfile_with_extension, tmpfile, strerror(errno));
         status = FC_EDBDIRACCESS;
         goto done;
     }
 
     if (cvd->version < remoteVersion) {
-        if (cvd->version == remoteVersion - 1) {
-            logg("*The %s database downloaded from %s is one version older than advertised in the DNS TXT record.\n",
-                 cvdfile,
-                 server);
-
-            /*
-             * Tolerate an off-by-one version mismatch.
-             * Chances are the new version was just published and the CDN is still updating.
-             */
-            status = FC_SUCCESS;
-            goto done;
-        } else {
-            logg("!The %s database downloaded from %s is more than one version older than the version advertised in the DNS TXT record.\n",
-                 cvdfile,
-                 server);
-            status = FC_EMIRRORNOTSYNC;
-            goto done;
-        }
+        logg("*The %s database downloaded from %s is older than the version advertised in the DNS TXT record.\n",
+             cvdfile,
+             server);
+        status = FC_EMIRRORNOTSYNC;
+        goto done;
     }
 
     status = FC_SUCCESS;
@@ -1144,7 +1472,10 @@ done:
     if (NULL != url) {
         free(url);
     }
-    if (FC_SUCCESS != status) {
+    if (
+        (FC_SUCCESS != status) &&
+        (FC_EMIRRORNOTSYNC != status) /* Keep older version, it's better than nothing. */
+    ) {
         if (NULL != tmpfile) {
             unlink(tmpfile);
         }
@@ -1281,7 +1612,7 @@ static fc_error_t downloadPatch(
         if (ret == FC_EEMPTYFILE) {
             logg("Empty script %s, need to download entire database\n", patch);
         } else {
-            logg("%cgetpatch: Can't download %s from %s\n", logerr ? '!' : '^', patch, url);
+            logg("%cdownloadPatch: Can't download %s from %s\n", logerr ? '!' : '^', patch, url);
         }
         status = ret;
         goto done;
@@ -1719,7 +2050,8 @@ static fc_error_t check_for_new_database_version(
     uint32_t *localVersion,
     uint32_t *remoteVersion,
     char **localFilename,
-    char **remoteFilename)
+    char **remoteFilename,
+    uint32_t *localTimestamp)
 {
     fc_error_t ret;
     fc_error_t status = FC_EARG;
@@ -1728,13 +2060,13 @@ static fc_error_t check_for_new_database_version(
     struct cl_cvd *local_database = NULL;
     char *remotename              = NULL;
 
-    uint32_t localver       = 0;
-    uint32_t localTimestamp = 0;
-    uint32_t remotever      = 0;
+    uint32_t localver  = 0;
+    uint32_t remotever = 0;
 
     if ((NULL == database) || (NULL == server) ||
         (NULL == localVersion) || (NULL == remoteVersion) ||
-        (NULL == localFilename) || (NULL == remoteFilename)) {
+        (NULL == localFilename) || (NULL == remoteFilename) ||
+        (NULL == localTimestamp)) {
         logg("!check_for_new_database_version: Invalid args!\n");
         goto done;
     }
@@ -1743,6 +2075,7 @@ static fc_error_t check_for_new_database_version(
     *remoteVersion  = 0;
     *localFilename  = NULL;
     *remoteFilename = NULL;
+    *localTimestamp = 0;
 
     /*
      * Check local database version (if exists)
@@ -1751,8 +2084,8 @@ static fc_error_t check_for_new_database_version(
         logg("*check_for_new_database_version: No local copy of \"%s\" database.\n", database);
     } else {
         logg("*check_for_new_database_version: Local copy of %s found: %s.\n", database, localname);
-        localTimestamp = local_database->stime;
-        localver       = local_database->version;
+        *localTimestamp = local_database->stime;
+        localver        = local_database->version;
     }
 
     /*
@@ -1760,7 +2093,7 @@ static fc_error_t check_for_new_database_version(
      */
     ret = query_remote_database_version(
         database,
-        localTimestamp,
+        *localTimestamp,
         dnsUpdateInfo,
         server,
         bPrivateMirror,
@@ -1778,15 +2111,15 @@ static fc_error_t check_for_new_database_version(
                      database, localver, remotever);
                 break;
             }
-            // else: Fall-through to Up-to-date case.
+            /* fall-through */
         }
         case FC_UPTODATE: {
             if (NULL == local_database) {
-                logg("!check_for_new_database_version: server claims we're up to date, but we don't have a local database!\n");
+                logg("!check_for_new_database_version: server claims we're up-to-date, but we don't have a local database!\n");
                 status = FC_EFAILEDGET;
                 goto done;
             }
-            logg("%s database is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n",
+            logg("%s database is up-to-date (version: %d, sigs: %d, f-level: %d, builder: %s)\n",
                  localname,
                  local_database->version,
                  local_database->sigs,
@@ -1798,6 +2131,12 @@ static fc_error_t check_for_new_database_version(
             remotever = localver;
             break;
         }
+        case FC_EFORBIDDEN: {
+            /* We tried to look up the version using HTTP and were actively blocked. */
+            logg("!check_for_new_database_version: Blocked from using server %s.\n", server);
+            status = FC_EFORBIDDEN;
+            goto done;
+        }
         default: {
             logg("!check_for_new_database_version: Failed to find %s database using server %s.\n", database, server);
             status = FC_EFAILEDGET;
@@ -1858,11 +2197,12 @@ fc_error_t updatedb(
 
     struct cl_cvd *cvd = NULL;
 
-    uint32_t localVersion  = 0;
-    uint32_t remoteVersion = 0;
-    char *localFilename    = NULL;
-    char *remoteFilename   = NULL;
-    char *newLocalFilename = NULL;
+    uint32_t localTimestamp = 0;
+    uint32_t localVersion   = 0;
+    uint32_t remoteVersion  = 0;
+    char *localFilename     = NULL;
+    char *remoteFilename    = NULL;
+    char *newLocalFilename  = NULL;
 
     char *tmpdir  = NULL;
     char *tmpfile = NULL;
@@ -1892,7 +2232,8 @@ fc_error_t updatedb(
                            &localVersion,
                            &remoteVersion,
                            &localFilename,
-                           &remoteFilename))) {
+                           &remoteFilename,
+                           &localTimestamp))) {
         logg("*updatedb: %s database update failed.\n", database);
         status = ret;
         goto done;
@@ -1914,8 +2255,21 @@ fc_error_t updatedb(
         /*
          * Download entire file.
          */
-        ret = getcvd(remoteFilename, tmpfile, server, remoteVersion, logerr);
-        if (FC_SUCCESS != ret) {
+        ret = getcvd(remoteFilename, tmpfile, server, localTimestamp, remoteVersion, logerr);
+        if (FC_UPTODATE == ret) {
+            logg("^Expected newer version of %s database but the server's copy is not newer than our local file (version %d).\n", database, localVersion);
+            if (NULL != localFilename) {
+                /* Received a 304 (not modified), must be up-to-date after all */
+                *dbFilename = cli_strdup(localFilename);
+            }
+            goto up_to_date;
+        } else if (FC_EMIRRORNOTSYNC == ret) {
+            /* Let's accept this older version, but keep the error code.
+             * We'll have fc_update_database() retry using CDIFFs.
+             */
+            logg("*Received an older %s CVD than was advertised. We'll keep it and try updating to the latest version with CDIFFs.\n", database);
+            status = ret;
+        } else if (FC_SUCCESS != ret) {
             status = ret;
             goto done;
         }
@@ -1925,7 +2279,8 @@ fc_error_t updatedb(
         /*
          * Attempt scripted/CDIFF incremental update.
          */
-        ret = FC_SUCCESS;
+        ret                         = FC_SUCCESS;
+        uint32_t numPatchesReceived = 0;
 
         tmpdir = cli_gentemp(g_tempDirectory);
         if (!tmpdir) {
@@ -1966,11 +2321,19 @@ fc_error_t updatedb(
                     break;
                 }
             }
-            if (FC_SUCCESS != ret)
+            if (FC_SUCCESS == ret) {
+                numPatchesReceived += 1;
+            } else {
                 break;
+            }
         }
 
-        if (FC_SUCCESS != ret) {
+        if (
+            (FC_EEMPTYFILE == ret) ||                                 /* Request a new CVD if we got an empty CDIFF.      */
+            (FC_SUCCESS != ret && (                                   /* Or if the incremental update failed:             */
+                                   (0 == numPatchesReceived) &&       /* 1. Ask for the CVD if we didn't get any patches, */
+                                   (localVersion < remoteVersion - 1) /* 2. AND if we're more than 1 version out of date. */
+                                   ))) {
             /*
              * Incremental update failed or intentionally disabled.
              */
@@ -1980,17 +2343,34 @@ fc_error_t updatedb(
                 logg("^Incremental update failed, trying to download %s\n", remoteFilename);
             }
 
-            ret = getcvd(remoteFilename, tmpfile, server, remoteVersion, logerr);
+            ret = getcvd(remoteFilename, tmpfile, server, localTimestamp, remoteVersion, logerr);
             if (FC_SUCCESS != ret) {
-                status = ret;
-                goto done;
+                if (FC_EMIRRORNOTSYNC == ret) {
+                    /* Note: We can't retry with CDIFF's if FC_EMIRRORNOTSYNC happened here.
+                     * If we did there could be an infinite loop.
+                     * Best option is to accept the older CVD.
+                     */
+                    logg("^Received an older %s CVD than was advertised. Incremental updates either failed or are disabled, so we'll have to settle for a slightly out-of-date database.\n", database);
+                    status = FC_SUCCESS;
+                } else {
+                    status = ret;
+                    goto done;
+                }
             }
 
             newLocalFilename = cli_strdup(remoteFilename);
+        } else if (0 == numPatchesReceived) {
+            logg("The database server doesn't have the latest patch for the %s database (version %u). The server will likely have updated if you check again in a few hours.\n", database, remoteVersion);
+            goto up_to_date;
         } else {
             /*
              * CDIFFs downloaded; Use CDIFFs to turn old CVD/CLD into new updated CLD.
              */
+            if (numPatchesReceived < remoteVersion - localVersion) {
+                logg("Downloaded %u patches for %s, which is fewer than the %u expected patches.\n", numPatchesReceived, database, remoteVersion - localVersion);
+                logg("We'll settle for this partial-update, at least for now.\n");
+            }
+
             size_t newLocalFilenameLen = 0;
             if (FC_SUCCESS != buildcld(tmpdir, database, tmpfile, g_bCompressLocalDatabase)) {
                 logg("!updatedb: Incremental update failed. Failed to build CLD.\n");
@@ -2043,8 +2423,8 @@ fc_error_t updatedb(
      */
 #ifdef _WIN32
     if (!access(newLocalFilename, R_OK) && unlink(newLocalFilename)) {
-        logg("!updatedb: Can't delete old database %s. Please fix the problem manually and try again.\n", newLocalFilename);
-        status = FC_EEMPTYFILE;
+        logg("!Update failed. Can't delete the old database %s to replace it with a new database. Please fix the problem manually and try again.\n", newLocalFilename);
+        status = FC_EDBDIRACCESS;
         goto done;
     }
 #endif
@@ -2073,7 +2453,7 @@ fc_error_t updatedb(
     if (flevel < cvd->fl) {
         logg("^Your ClamAV installation is OUTDATED!\n");
         logg("^Current functionality level = %d, recommended = %d\n", flevel, cvd->fl);
-        logg("DON'T PANIC! Read https://www.clamav.net/documents/installing-clamav\n");
+        logg("DON'T PANIC! Read https://docs.clamav.net/manual/Installing.html\n");
     }
 
     *signo      = cvd->sigs;
@@ -2087,7 +2467,9 @@ fc_error_t updatedb(
 
 up_to_date:
 
-    status = FC_SUCCESS;
+    if (status != FC_EMIRRORNOTSYNC) {
+        status = FC_SUCCESS;
+    }
 
 done:
 
@@ -2176,7 +2558,7 @@ fc_error_t updatecustomdb(
         remote_dbtime = statbuf.st_mtime;
         dbtime        = (CLAMSTAT(databaseName, &statbuf) != -1) ? statbuf.st_mtime : 0;
         if (dbtime > remote_dbtime) {
-            logg("%s is up to date (version: custom database)\n", databaseName);
+            logg("%s is up-to-date (version: custom database)\n", databaseName);
             goto up_to_date;
         }
 
@@ -2203,7 +2585,7 @@ fc_error_t updatecustomdb(
 
         ret = downloadFile(url, tmpfile, 1, logerr, dbtime);
         if (ret == FC_UPTODATE) {
-            logg("%s is up to date (version: custom database)\n", databaseName);
+            logg("%s is up-to-date (version: custom database)\n", databaseName);
             goto up_to_date;
         } else if (ret > FC_UPTODATE) {
             logg("%cCan't download %s from %s\n", logerr ? '!' : '^', databaseName, url);
@@ -2228,7 +2610,7 @@ fc_error_t updatecustomdb(
         }
         snprintf(tmpfile_with_extension, tmpfile_with_extension_len + 1, "%s-%s", tmpfile, databaseName);
         if (rename(tmpfile, tmpfile_with_extension) == -1) {
-            logg("!updatecustomdb: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+            logg("!Custom database update failed: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
             free(tmpfile_with_extension);
             status = FC_EDBDIRACCESS;
             goto done;
@@ -2251,8 +2633,8 @@ fc_error_t updatecustomdb(
      */
 #ifdef _WIN32
     if (!access(databaseName, R_OK) && unlink(databaseName)) {
-        logg("!updatecustomdb: Can't delete old database %s. Please fix the problem manually and try again.\n", databaseName);
-        status = FC_EEMPTYFILE;
+        logg("!Custom database update failed. Can't delete the old database %s to replace it with a new database. Please fix the problem manually and try again.\n", databaseName);
+        status = FC_EDBDIRACCESS;
         goto done;
     }
 #endif
@@ -2281,7 +2663,7 @@ fc_error_t updatecustomdb(
         if (flevel < cvd->fl) {
             logg("^Your ClamAV installation is OUTDATED!\n");
             logg("^Current functionality level = %d, recommended = %d\n", flevel, cvd->fl);
-            logg("DON'T PANIC! Read https://www.clamav.net/documents/installing-clamav\n");
+            logg("DON'T PANIC! Read https://docs.clamav.net/manual/Installing.html\n");
         }
 
         cl_cvdfree(cvd);
