@@ -178,12 +178,12 @@ my %CONF = (
     'NOBZ2PREFIX' => -1,
     'NO_FD_SET' => -1,
     'PACKAGE' => 'PACKAGE_NAME',
-    'PACKAGE_BUGREPORT' => '"https://bugzilla.clamav.net/"',
+    'PACKAGE_BUGREPORT' => '"https://github.com/Cisco-Talos/clamav/issues"',
     'PACKAGE_NAME' => '"ClamAV"',
-    'PACKAGE_STRING' => '"ClamAV 0.103.3"',
+    'PACKAGE_STRING' => '"ClamAV 0.103.4"',
     'PACKAGE_TARNAME' => '"clamav"',
     'PACKAGE_URL' => '"https://www.clamav.net/"',
-    'PACKAGE_VERSION' => '"0.103.3"',
+    'PACKAGE_VERSION' => '"0.103.4"',
     'SCANBUFF' => '131072',
     'SETPGRP_VOID' => '1',
     'SIZEOF_INT' => '4',
